/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.interceptor;

import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.reflect.Method;

import org.t2framework.commons.annotation.composite.SingletonScope;
import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.aop.spi.Invocation;
import org.t2framework.commons.util.Logger;
import org.t2framework.lucy.annotation.core.Inject;

/**
 * 
 * {@.en Simple performance tracking interceptor.}
 * 
 * <br />
 * 
 * {@.ja 呼び出しを追跡可能で簡単なパフォーマンス測定用の{@link Interceptor}です.}
 * 
 * @author ryushi
 * @author shot
 * 
 */
@SingletonScope
public class CallFlowTrackingInterceptor implements Interceptor {

	public static final String CF_CATEGORY = "org.t2framework.lucy.profiler.callflow";

	static final Logger LOG = Logger.getLogger(CF_CATEGORY);

	static final MemoryMXBean mmb = ManagementFactory.getMemoryMXBean();

	protected String beginMessageId = "DCommonInterceptor0003";

	protected String endMessageId = "DCommonInterceptor0004";

	@Override
	public Object intercept(Invocation<Method> invocation) throws Throwable {
		if (LOG.isDebugEnabled()) {
			Method mtd = invocation.getType();
			long start = System.currentTimeMillis();
			try {
				// thread id,class name,method name,
				// heap usage,nonHeap usage
				Object[] param = new Object[] { Thread.currentThread().getId(),
						mtd.getDeclaringClass().getName(), mtd.getName(),
						mmb.getHeapMemoryUsage().getUsed(),
						mmb.getNonHeapMemoryUsage().getUsed() };
				LOG.log(getBeginMessageId(), param);

				return invocation.proceed();
			} finally {
				Object[] param = new Object[] { Thread.currentThread().getId(),
						mtd.getDeclaringClass(), mtd.getName(),
						mmb.getHeapMemoryUsage().getUsed(),
						mmb.getNonHeapMemoryUsage().getUsed(),
						System.currentTimeMillis() - start };
				LOG.log(getEndMessageId(), param);
			}
		} else {
			return invocation.proceed();
		}
	}

	public String getBeginMessageId() {
		return beginMessageId;
	}

	@Inject
	public void setBeginMessageId(String beginMessageId) {
		this.beginMessageId = beginMessageId;
	}

	public String getEndMessageId() {
		return endMessageId;
	}

	@Inject
	public void setEndMessageId(String endMessageId) {
		this.endMessageId = endMessageId;
	}
}
