/*
 * Copyright 2008-2009 the T2 Project ant the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.t2framework.lucy.interceptor;

import java.lang.reflect.Method;

import org.t2framework.commons.annotation.composite.SingletonScope;
import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.aop.spi.Invocation;
import org.t2framework.commons.util.Logger;
import org.t2framework.lucy.annotation.core.Inject;

/**
 * 
 * {@.en Logging interceptor.}
 * 
 * <br />
 * 
 * {@.ja }
 * 
 * @author shot
 * 
 */
@SingletonScope
public class LoggingInterceptor implements Interceptor {

	protected static Logger logger = Logger.getLogger(LoggingInterceptor.class);

	protected String startMessageCode = "DCommonInterceptor0001";

	protected String endMessageCode = "DCommonInterceptor0002";

	@Override
	public Object intercept(Invocation<Method> invocation) throws Throwable {
		try {
			log(getStartMessageCode(), invocation);
			return invocation.proceed();
		} catch (Throwable e) {
			throw e;
		} finally {
			log(getEndMessageCode(), invocation);
		}
	}

	@Inject
	public void setStartMessageCode(String startMessaCode) {
		this.startMessageCode = startMessaCode;
	}

	@Inject
	public void setEndMessageCode(String endMessageCode) {
		this.endMessageCode = endMessageCode;
	}

	public String getStartMessageCode() {
		return startMessageCode;
	}

	public String getEndMessageCode() {
		return endMessageCode;
	}

	protected void log(String messageCode, Invocation<Method> invocation) {
		final StringBuilder builder = new StringBuilder();
		Object[] args = invocation.getArguments();
		if (args != null && 0 < args.length) {
			for (Object o : args) {
				builder.append(o);
				builder.append(", ");
			}
			if (2 < builder.length()) {
				builder.setLength(builder.length() - 2);
			}
		}
		getLogger().log(
				messageCode,
				new Object[] { invocation.getThis().getClass(),
						invocation.getType().getName(), new String(builder) });
	}

	protected Logger getLogger() {
		return logger;
	}
}
